/*
 * fluid.c --- stokes η (ή®) βĻ벽
 *   written by Shioya, modified by mk
 *   The original program is fluid.c by T.Kudo.
 */

#include <math.h>
#include <stdio.h>
#define G_DOUBLE
#include <glsc.h>
#include <matrix.h>

void glsc(int U, int T, int **num, int *cum, vector, vector, vector);
void domain(int T, int **num, vector x, vector y);
void flow(int U, int *cum, vector x, vector y, vector u);
void arrow(double x1, double y1, double x2, double y2);

int main(int argc, char **argv)
{
    FILE *f1, *f2;
    int i, j, U, T, SIZE;
    int *cum, **num;
    vector x, y, u;
    double h;

    if (argc != 3) {
        fprintf(stderr, "usage: %s ΰǡ ׻ǡ\n", argv[0]);
        exit(1);
    }
    if ((f1 = fopen(argv[1], "r")) == NULL) {
        fprintf(stderr, "Can't open %s", argv[1]);
        exit(2);
    }
    if ((f2 = fopen(argv[2], "r")) == NULL) {
        fprintf(stderr, "Can't open %s", argv[2]);
        exit(3);
    }

    fscanf(f1, "%d %d %lf", &T, &U, &h);

    x = new_vector(U);
    y = new_vector(U);
    for (i = 0; i < U; i++)
        fscanf(f1, "%lf %lf", &x[i], &y[i]);

    num = (int **) malloc(T * sizeof(void *));
    if (num == NULL) {
        printf("ǤбѤΥ꡼­ޤ\n");
	exit(4);
    }
    for (i = 0; i < T; i++) {
        num[i] = (int *) malloc(sizeof(int) * 9);
        if (num[i] == NULL) {
            printf("ǤбѤΥ꡼­ޤ\n");
	    exit(4);
        }
    }
    for (i = 0; i < T; i++)
        for (j = 0; j < 9; j++)
            fscanf(f1, "%d", &num[i][j]);

    if ((cum = (int *) malloc(sizeof(int) * U)) == NULL) {
        printf("ֹѤΥ꡼­ޤ \n");
	exit(4);
    }
    for (i = 0; i < U; i++)
        fscanf(f1, "%d", &cum[i]);

    fscanf(f1, "%d", &SIZE);
    fclose(f1);

    u = new_vector(SIZE);
    for (i = 0; i < SIZE; i++)
        fscanf(f2, "%lf", &u[i]);
    fclose(f2);

    glsc(U, T, num, cum, x, y, u);

    return 0;
}

void glsc(int U, int T, int **num, int *cum,
	  vector x, vector y, vector u)
{
    int i;
    double left, right, top, bottom, widx, widy, Lx, Ly;

    left = right = x[0];
    top = bottom = y[0];

    for (i = 1; i < U; i++) {
        if (left > x[i])
            left = x[i];
        else if (right < x[i])
            right = x[i];
        if (top < y[i])
            top = y[i];
        else if (bottom > y[i])
            bottom = y[i];
    }

    widx = fabs(left) + fabs(right);
    widy = fabs(top) + fabs(bottom);

    left -= 0.1 * widx;
    right += 0.1 * widx;
    top += 0.1 * widy;
    bottom -= 0.1 * widy;

    widx = fabs(left) + fabs(right);
    widy = fabs(top) + fabs(bottom);

    if (widx >= widy) {
        Lx = 150.0;
        Ly = Lx * widy / widx;
    }
    else {
        Ly = 150.0;
        Lx = Ly * widx / widy;
    }

    g_init("Shioya", Lx + 10.0, Ly + 10.0);
    g_device(G_BOTH);
    g_def_scale(0, left, right, bottom, top, 5.0, 5.0, Lx, Ly);
    g_cls();
    g_sel_scale(0);

    printf("ֹ :  \n");
    printf("       \n");
    printf("       \n");
    printf("طʤοֹϡ ");
    scanf("%d", &i);
    g_line_color(i);
    g_area_color(i);
    g_box(left, right, bottom, top, 1, 1);

    printf("call domain\n");
    domain(T, num, x, y);
    printf("call flow\n");
    flow(U, cum, x, y, u);

    g_sleep(-1.0);
    g_term();
}

void domain(int T, int **num, vector x, vector y)
{
    int i, j;
    double zx[4], zy[4];

    printf("ΰοֹϡ ");
    scanf("%d", &i);

    g_def_line(0, i, 0, 4);
    g_sel_line(0);
    g_area_color(i);

    for (i = 0; i < T; i++) {
        for (j = 0; j < 4; j++) {
            zx[j] = x[num[i][j]];
            zy[j] = y[num[i][j]];
        }
        g_polygon(zx, zy, 4, 0, 1);
        g_polyline(zx, zy, 4);
    }
}

void flow(int U, int *cum, vector x, vector y, vector u)
{
    int i;
    double d;

    printf("οֹϡ ");
    scanf("%d", &i);
    g_def_line(0, i, 0, 0);
    g_sel_line(0);

    printf("Ĺϡ ");
    scanf("%lf", &d);

    for (i = 0; i < U; i++)
        arrow(x[i], y[i], x[i] + d * u[cum[i]], y[i] + d * u[cum[i] + 1]);
}

void arrow(double x1, double y1, double x2, double y2)
{
    double x3, y3, x4, y4;

    x3 = 0.1 * ((x1 + 9.0 * x2) + (y2 - y1));
    x4 = 0.1 * ((x1 + 9.0 * x2) - (y2 - y1));
    y3 = 0.1 * ((y1 + 9.0 * y2) - (x2 - x1));
    y4 = 0.1 * ((y1 + 9.0 * y2) + (x2 - x1));

    g_move(x1, y1);
    g_plot(x2, y2);
    g_plot(x3, y3);
    g_move(x2, y2);
    g_plot(x4, y4);
}
